@@ -77,7 +77,7 @@ class Page(MPTTModel):
     updated = models.DateTimeField(_('updated'), auto_now=True)
     parent = models.ForeignKey('self', null=True, blank=True, related_name='subpages', verbose_name=_('parent'))
     # TODO: add keywords, description (as meta?)
-    title = models.CharField(_('title'), blank=True, max_length=255)
+    title = models.CharField(_('title'), max_length=255)
     url = FiberURLField(blank=True)
     redirect_page = models.ForeignKey('self', null=True, blank=True, related_name='redirected_pages', verbose_name=_('redirect page'), on_delete=models.SET_NULL)
     mark_current_regexes = models.TextField(_('mark current regexes'), blank=True)
@@ -309,6 +309,12 @@ class Image(models.Model):
     def get_image_information(self):
         self.width, self.height = get_image_dimensions(self.image) or (0, 0)
 
+    def get_filename(self):
+        return os.path.basename(self.image.name)
+
+    def get_size(self):
+        return '%s x %d' % (self.width, self.height)
+
 
 class File(models.Model):
     created = models.DateTimeField(_('created'), auto_now_add=True)
@@ -335,3 +341,6 @@ class File(models.Model):
     def delete(self, *args, **kwargs):
         os.remove(os.path.join(settings.MEDIA_ROOT, str(self.file)))
         super(File, self).delete(*args, **kwargs)
+
+    def get_filename(self):
+        return os.path.basename(self.file.name)
