@@ -630,6 +630,13 @@ var BaseFileSelectDialog = AdminRESTDialog.extend({
 		return 'file';
 	},
 
+	uneditables_formatter: function(value, row_data) {
+		if (!row_data.can_edit) {
+			return '<span class="non-editable">' + value + '</span>';
+		}
+			return value;
+	},
+
 	create_upload_button: function() {
 		var button_pane = this.uiDialog.parent().find('.ui-dialog-buttonpane');
 
@@ -662,9 +669,9 @@ var BaseFileSelectDialog = AdminRESTDialog.extend({
 			element: upload_button_pane[0],
 			button: upload_button_pane[0], // connecting directly to the jQUery UI upload_button doesn't work
 			action: this.get_upload_path(),
-			params: {
-				title: 'uploaded'
-			},
+			onSubmit: $.proxy(function(id, fileName) {
+				uploader._options.params.title = fileName;
+			}, this),
 			onComplete: $.proxy(function(id, fileName, responseJSON) {
 				this.refresh_grid();
 			}, this),
@@ -710,9 +717,14 @@ var BaseFileSelectDialog = AdminRESTDialog.extend({
 
 		delete_button.addClass('ui-button-disabled ui-state-disabled');
 
-		this.select_grid.bind('datagrid.select', function() {
-			delete_button.attr('disabled', '');
-			delete_button.removeClass('ui-button-disabled ui-state-disabled');
+		this.select_grid.bind('datagrid.select', function(e) {
+			if (e.row.can_edit){
+				delete_button.attr('disabled', '');
+				delete_button.removeClass('ui-button-disabled ui-state-disabled');
+			}
+			else if (!delete_button.hasClass('ui-button-disabled ui-state-disabled')){
+					delete_button.addClass('ui-button-disabled ui-state-disabled');
+				}
 		});
 
 		var self = this;
@@ -787,12 +799,13 @@ Fiber.ImageSelectDialog = BaseFileSelectDialog.extend({
 		this.select_grid.simple_datagrid({
 			columns: [
 				{title: gettext('Image'), key: 'image', on_generate: thumbnail_formatter},
-				{title: gettext('Filename'), key: 'filename'},
+				{title: gettext('Filename'), key: 'filename', on_generate: this.uneditables_formatter},
 				{title: gettext('Size'), key: 'size'},
 				{title: gettext('Updated'), key: 'updated'}
 			],
 			url: this.options.url,
-			order_by: 'filename'
+			order_by: 'updated',
+			sortorder: 'desc'
 		});
 
 		this.select_grid.bind('datagrid.select', function() {
@@ -868,11 +881,12 @@ Fiber.FileSelectDialog = BaseFileSelectDialog.extend({
 
 		this.select_grid.simple_datagrid({
 			columns: [
-				{title: gettext('Filename'), key: 'filename'},
+				{title: gettext('Filename'), key: 'filename', on_generate: this.uneditables_formatter},
 				{title: gettext('Updated'), key: 'updated'}
 			],
 			url: this.options.url,
-			order_by: 'filename'
+			order_by: 'updated',
+			sortorder: 'desc'
 		});
 		this.select_grid.bind('datagrid.select', function() {
 			action_button.attr('disabled', '');
@@ -913,7 +927,7 @@ Fiber.PageSelectDialog = AdminRESTDialog.extend({
 	defaults: {
 		url: '/api/v2/pages/',
 		width: 480,
-		height: 320,
+		height: 520,
 		start_width: 480,
 		start_height: 320
 	},
@@ -961,7 +975,7 @@ Fiber.PageSelectDialog = AdminRESTDialog.extend({
 
 		function createLi(node, $li) {
 			if (node.change_url) {
-				$li.find('.title').before('<span class="icon"></span>');
+				$li.find('.jqtree-title').before('<span class="icon"></span>');
 				$li.find('div').addClass('page');
 			}
 		}
@@ -1407,7 +1421,7 @@ var adminPage = {
 		function createLi(node, $li) {
 			if (node.change_url) {
 				var $div = $li.find('div');
-				$li.find('.title').before('<span class="icon"></span>');
+				$li.find('.jqtree-title').before('<span class="icon"></span>');
 				$div.addClass('page');
 
 				if (!node.show_in_menu) {
@@ -1415,17 +1429,21 @@ var adminPage = {
 				}
 
 				if (!node.is_public) {
-					$div.addClass('not-public');
+					$div.addClass('non-public');
 				}
 
 				if (node.is_redirect) {
 					$div.addClass('redirect');
 				}
+
+				if (!node.editable) {
+					$div.addClass('non-editable');
+				}
 			}
 		}
 
 		function canMove(node) {
-			if (node.url) {
+			if (node.url && node.editable) {
 				return true;
 			}
 			else {
@@ -1435,6 +1453,9 @@ var adminPage = {
 		}
 
 		function canMoveTo(moved_node, target_node, position) {
+			if (!target_node.editable) {
+				return false;
+			}
 			if (!target_node.url) {
 				// can move inside menu, not before or after
 				return (position == 'inside');
@@ -1511,6 +1532,9 @@ var adminPage = {
 		$(document.body).find('.ui-context-menu').remove();
 
 		var node = e.node;
+		if (!node.editable){
+			return;
+		}
 
 		var contextmenu = $('<ul class="ui-context-menu"></ul>');
 
@@ -1850,7 +1874,7 @@ function reloadPage(params) {
 			url: '/api/v2/pages/' + page_id + '/',
 			type: 'GET',
 			success: function(data) {
-				window.location.replace(data.url);
+				window.location.replace(data.page_url);
 			},
 			error: function() {
 				if (params && params.error) {
@@ -1894,6 +1918,9 @@ Fiber.FiberItem = Class.extend({
 	},
 
 	show_admin_element: function() {
+		if (!this.element_data.can_edit) {
+			return;
+		}
 		if (!this.button) {
 			this.create_button();
 		}
@@ -2146,6 +2173,9 @@ Fiber.FiberItem = Class.extend({
 	on_contextmenu: function(e) {
 		e.preventDefault();
 		e.stopPropagation();
+		if (!this.element_data.can_edit) {
+			return;
+		}
 
 		// remove other visible context menus
 		$(document.body).find('.ui-context-menu').remove();
